package framework.everywaretechnologies.it.icarus;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.util.Log;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.Volley;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Consumer;


public class Icarus {

    enum Categories{ //categorie di evento

        APPLICATION,ACTION,NOTIFICATION;
    }

    private final static String ICARUS_URL="https://webdev.ewlab.di.unimi.it/icarus/str";

    private static final Icarus ourInstance = new Icarus();
    private Queue<JSONObject> pendingResources; //FIFO
    private Lock pendingResourceLock;
    private Context appContext;// serve per istanziare un un'unica coda di richieste Volley (e non averne una per Activity)
    private static Context curContext; //context dell'Activity corrente
    private RequestQueue mVolleyRequestQueue;

    public static Icarus getInstance(Context c) {

        curContext=c;
        return ourInstance;
    }

    private Icarus() {

        pendingResources=new LinkedList<>();
        pendingResourceLock=new ReentrantLock();
    }

    public void setApplicationContext(Context applicationContext){

        appContext=applicationContext;

        //adesso che ho l'applicationContext dell'app chiamante creo la coda di richieste per Volley
        mVolleyRequestQueue= Volley.newRequestQueue(appContext);
    }

    //----------------- LOG PREFERENCE CHANGES -----------------------

    public void logUserSettings(UserSettingsResource userSettingsResource){

        Log.d(curContext.getString(R.string.icarus_class),"log pref changes");

        JSONObject prefsChangesToLog=userSettingsResource.getUserSettingsChangesInJSONFormat();
        postIcarusHttpRequest(prefsChangesToLog);
        userSettingsResource.clear();
    }

    //---------------------- LOG SCREEN TRACKING ------------------------------


    public void trackScreen(Context c,String screenEventType){

        Log.d(curContext.getString(R.string.icarus_class),"log screen track");

        String screenName=c.getClass().getSimpleName();

        if(screenEventType.equalsIgnoreCase(ScreenTrackResource.EVENT_ENTERED) || screenEventType.equalsIgnoreCase(ScreenTrackResource.EVENT_EXITED) ){

            IcarusResource resource=new ScreenTrackResource(screenName,screenEventType);

            Gson gson=new Gson();
            String screenTrackJson=gson.toJson(resource);
            try {

                JSONObject icarusPayload=new JSONObject(screenTrackJson);
                postIcarusHttpRequest(icarusPayload);

                Log.d(curContext.getString(R.string.icarus_class),"log screen track -> Volley");

            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    }


    //---------------------- LOG EVENT TRACKING --------------------------

    public void logEvent(String eventName,String category,String label){ //TODO: ci saranno altri parametri da passargli come quello dell'attività

        Log.d(curContext.getString(R.string.icarus_class),"log event");

    }




    //--------------------------- UTILITY ----------------------------------
    private void postIcarusHttpRequest(final JSONObject icarusPayload){

        final JsonObjectRequest jsonObjectRequest=new JsonObjectRequest(Request.Method.POST, ICARUS_URL, icarusPayload, new Response.Listener<JSONObject>() {
            @Override
            public void onResponse(JSONObject response) {

                Log.d(curContext.getString(R.string.icarus_class),"onVolley response POSITIVE");

                //controllo se ho qualche elemento che precedentemente non sono riuscito ad inviare e nel caso lo provo ad inviare
                pendingResourceLock.lock();

                if(pendingResources.peek() != null){ //se ho qualche elemento pendente da inviare

                    Log.d(curContext.getString(R.string.icarus_class),"messaggio pendente");
                    JSONObject pendingResource=pendingResources.poll();
                    postIcarusHttpRequest(pendingResource);
                }
                pendingResourceLock.unlock();

            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {

                Log.d(curContext.getString(R.string.icarus_class),"onVolley response NEGATIVE");

                //se per qualche motivo la richiesta http non va a buon fine, salvo il payload che avrei dovuto mandare in cache
                pendingResourceLock.lock();
                pendingResources.add(icarusPayload);
                pendingResourceLock.unlock();
            }
        });

        mVolleyRequestQueue.add(jsonObjectRequest);//faccio effettivamente la richiesta
    }

    //SharedPreferences utility methods
    private void editIntPref(SharedPreferences settings,String nomePref,int newValue){

        SharedPreferences.Editor editor=settings.edit();
        editor.putInt(nomePref,newValue);
        editor.commit();
    }

    private void editBoolPref(SharedPreferences settings,String nomePref,boolean newValue){

        SharedPreferences.Editor editor=settings.edit();
        editor.putBoolean(nomePref,newValue);
        editor.commit();
    }
}
